"use strict";

function _typeof(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var types = _interopRequireWildcard(require("@babel/types"));

var _isValidPath = _interopRequireDefault(require("is-valid-path"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function _getRequireWildcardCache() { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || _typeof(obj) !== "object" && typeof obj !== "function") { return { "default": obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj["default"] = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function findOptionFromSource(source, state) {
  var opts = state.opts;
  if (opts[source]) return source;
  var opt = Object.keys(opts).find(function (key) {
    return !(0, _isValidPath["default"])(key) && new RegExp(key).test(source);
  });
  if (opt) return opt;
}

function getMatchesFromSource(opt, source) {
  var regex = new RegExp(opt, 'g');
  var matches = [];
  var m;

  while ((m = regex.exec(source)) !== null) {
    if (m.index === regex.lastIndex) regex.lastIndex++;
    m.forEach(function (match) {
      return matches.push(match);
    });
  }

  return matches;
}

function transform(transformOption, importName, matches) {
  if (typeof transformOption === 'function') {
    return transformOption(importName, matches);
  }

  return transformOption.replace(/\$\{\s?([\w\d]*)\s?\}/ig, function (str, g1) {
    if (g1 === 'member') return importName;
    return matches[g1];
  });
}
/**
 * Camel conversion horizontal line interval
 * @param {String} name 
 * 
 * `CopyToClipboard` => `copy-to-clipboard`
 */


function toLine(name) {
  return name.replace(/\B([A-Z])/g, '-$1').toLowerCase();
}

var plugName = 'transform-uiw-import';

var _default = function _default() {
  return {
    name: plugName,
    visitor: {
      // https://babeljs.io/docs/en/babel-types#importdeclaration
      ImportDeclaration: function ImportDeclaration(path, state) {
        // https://github.com/babel/babel/tree/master/packages/babel-types#timportdeclarationspecifiers-source
        // path.node has properties 'source' and 'specifiers' attached.
        // path.node.source is the library/module name, aka 'uiw'.
        // path.node.specifiers is an array of ImportSpecifier | ImportDefaultSpecifier | ImportNamespaceSpecifier
        var uiwConf = {
          // transform: 'uiw/lib/cjs/${member}',
          transform: function transform(importName) {
            return "uiw/lib/cjs/".concat(toLine(importName));
          }
        };
        state.opts.uiw = state.opts.uiw ? _objectSpread({}, uiwConf, {}, state.opts.uiw) : uiwConf;
        var node = path.node;
        var source = node.source,
            specifiers = node.specifiers;
        var opt = findOptionFromSource(source.value, state); // aka 'uiw'

        var isRegexp = opt && !(0, _isValidPath["default"])(opt);
        var opts = state.opts[opt]; // this options

        if (!opts) return;

        if (!opts.transform) {
          throw new Error("".concat(plugName, ": transform option is required for module '").concat(source.value, "'. "));
        }

        var transforms = [];
        var fullImports = path.node.specifiers.filter(function (specifier) {
          return specifier.type !== 'ImportSpecifier';
        });
        var memberImports = path.node.specifiers.filter(function (specifier) {
          return specifier.type === 'ImportSpecifier';
        });

        if (fullImports.length > 0) {
          // Examples of "full" imports:
          //      import * as name from 'module'; (ImportNamespaceSpecifier)
          //      import name from 'module'; (ImportDefaultSpecifier)
          if (opts.preventFullImport) {
            throw new Error("".concat(plugName, ": import of entire module '").concat(source.value, "' not allowed due to preventFullImport setting."));
          }

          if (memberImports.length > 0) {
            // Swap out the import with one that doesn't include member imports.  Member imports should each get their own import line
            // transform this:
            //      import Bootstrap, { Grid } from 'react-bootstrap';
            // into this:
            //      import Bootstrap from 'react-bootstrap';
            transforms.push(types.importDeclaration(fullImports, types.stringLiteral(source.value)));
          }
        }

        var matches = isRegexp ? getMatchesFromSource(opt, source.value) : [];
        memberImports.forEach(function (memberImport) {
          // Examples of member imports:
          //      import { member } from 'module'; (ImportSpecifier)
          //      import { member as alias } from 'module' (ImportSpecifier)
          // transform this:
          //      import { Grid as gird } from 'react-bootstrap';
          // into this:
          //      import gird from 'react-bootstrap/lib/Grid';
          // or this, if skipDefaultConversion = true:
          //      import { Grid as gird } from 'react-bootstrap/lib/Grid';
          var importName = memberImport.imported.name;
          var replace = transform(opts.transform, importName, matches);
          var newImportSpecifier = opts.skipDefaultConversion ? memberImport : types.importDefaultSpecifier(types.identifier(memberImport.local.name));
          transforms.push(types.importDeclaration([newImportSpecifier], types.stringLiteral(replace)));
        });

        if (transforms.length > 0) {
          path.replaceWithMultiple(transforms);
        }
      }
    }
  };
};

exports["default"] = _default;